// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package device

import (
	"context"
	"smart-auth/errors"
	"smart-auth/model"
	"smart-auth/pb/v1"

	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
)

func DeviceParamSet(ctx context.Context, req *v1.DeviceParamSetRequest) (reply *v1.DeviceParamSetReply, err error) {
	if req.Sn == "" || req.Param == "" {
		return nil, errors.ParamsError
	}

	p := &model.TDevice{}
	where := map[string]interface{}{
		"sn":          req.Sn,
		"device_code": req.DeviceCode,
	}

	values := map[string]interface{}{
		"param": req.Param,
	}

	// 查询设备是否入库
	err = p.Query(database.DB(), where)
	if err != nil {
		if err == gorm.ErrRecordNotFound {
			return nil, errors.DeviceNotExistError
		}
		return nil, errors.DataBaseError
	}

	// 更新参数
	err = p.UpdateSome(database.DB(), values)
	if err != nil {
		return nil, errors.DataBaseError
	}

	return &v1.DeviceParamSetReply{}, nil
}
